#include "PrecompiledHeader.h"

#include "main/Module_Main.h"

#include "main/Plugin.h"


#ifdef _WIN32
#	define dlerror() GetLastError()
#endif

using namespace Castor3D;
using namespace Castor::Utils;


Castor3D::Plugin :: Plugin( const String & p_fileName)
	:	m_pDLLRefCount			( NULL),
		m_pfnGetRequiredVersion	( 0),
		m_pfnRegisterPlugin		( 0),
		m_library				( new DynamicLibrary)
{

	Cout( CU_T( "Plugin name [") << p_fileName << CU_T( "]") << std::endl);

	if ( ! m_library->Open( p_fileName.char_str()))
	{
		Cerr( CU_T( "Error encountered while loading plugin : ") << dlerror() << std::endl);
		throw;
	}

	m_pfnGetRequiredVersion = reinterpret_cast <fnGetRequiredVersion *> ( m_library->GetFunction( "GetRequiredVersion"));

	if (m_pfnGetRequiredVersion == NULL)
	{
		Cerr( CU_T( "Error encountered while loading plugin factory : ") << dlerror() << std::endl);
		throw;
	}

	m_pfnRegisterPlugin = reinterpret_cast <fnRegisterPlugin *> ( m_library->GetFunction( "RegisterPlugin"));

	if (m_pfnRegisterPlugin == NULL)
	{
		Cerr( CU_T( "Error encountered while loading plugin destroyer : ") << dlerror() << std::endl);
		throw;
	}

	m_pDLLRefCount = new size_t( 1);
}



Castor3D::Plugin :: Plugin( const Castor3D::Plugin & p_plugin)
	:	m_pDLLRefCount			( p_plugin.m_pDLLRefCount),
		m_pfnGetRequiredVersion	( p_plugin.m_pfnGetRequiredVersion),
		m_pfnRegisterPlugin		( p_plugin.m_pfnRegisterPlugin)
{
	++*m_pDLLRefCount;
}



Castor3D::Plugin & Castor3D::Plugin :: operator =( const Castor3D::Plugin & p_plugin)
{
	m_pfnGetRequiredVersion = p_plugin.m_pfnGetRequiredVersion;
	m_pfnRegisterPlugin = p_plugin.m_pfnRegisterPlugin;
	m_pDLLRefCount = p_plugin.m_pDLLRefCount;
	++*m_pDLLRefCount;
	return *this;
}



Castor3D::Plugin :: ~Plugin()
{
	if (--*m_pDLLRefCount == 0)
	{
		delete m_pDLLRefCount;
		delete m_library;
	}
}


